from apiron.service.base import Service
from apiron.endpoint import JsonEndpoint
from apiron.client import ServiceCaller
from urllib.parse import urlunparse

__all__=['Orthanc']


class OrthancService(Service):
    def __init__(self, *args, **kwargs):
        scheme = kwargs.pop('scheme', 'http')
        domain = kwargs.pop('domain', 'localhost')
        port = kwargs.pop('port', 8042)
        self.domain = urlunparse((scheme, ':'.join((domain, str(port))), '', '', '', ''))
        super(*args, **kwargs)

    patients = JsonEndpoint(path='/patients')
    patient = JsonEndpoint(path='/patients/{id}')
    studies = JsonEndpoint(path='/studies')
    study = JsonEndpoint(path='/studies/{id}')
    series = JsonEndpoint(path='/series')
    part = JsonEndpoint(path='/series/{id}')
    instances = JsonEndpoint(path='/instances')
    instance = JsonEndpoint(path='/instances/{id}')
    instance_tag = JsonEndpoint(path='/instances/{id}/simplified-tags')
    changes = JsonEndpoint(path='/changes')
    queries = JsonEndpoint(path='/queries')
    find = JsonEndpoint(path='/tools/find', default_method='POST')

class Orthanc:
    """Interface to apiron

    """

    def __init__(self, *args, **kwargs):
        self.service = OrthancService(*args, **kwargs)

    @property
    def server(self):
        return self.service.domain

    def __repr__(self):
        return '<Orthanc server at {}>'.format(self.service.domain)

    def patients(self, **kwargs):
        return ServiceCaller.call(self.service, self.service.patients, **kwargs)

    def patient(self, id_, **kwargs):
        return ServiceCaller.call(self.service, self.service.patient, path_kwargs={'id': id_}, **kwargs)

    def studies(self, **kwargs):
        return ServiceCaller.call(self.service, self.service.studies, **kwargs)

    def study(self, id_, **kwargs):
        return ServiceCaller.call(self.service, self.service.study, path_kwargs={'id': id_}, **kwargs)

    def series(self, **kwargs):
        return ServiceCaller.call(self.service, self.service.series, **kwargs)

    def part(self, id_, **kwargs):
        return ServiceCaller.call(self.service, self.service.part, path_kwargs={'id': id_}, **kwargs)

    def instances(self, **kwargs):
        return ServiceCaller.call(self.service, self.service.instances, **kwargs)

    def instance(self, id_, **kwargs):
        return ServiceCaller.call(self.service, self.service.instance, path_kwargs={'id': id_}, **kwargs)

    def instance_tag(self, id_, **kwargs):
        return ServiceCaller.call(self.service, self.service.instance_tag, path_kwargs={'id': id_}, **kwargs)

    def changes(self, **kwargs):
        return ServiceCaller.call(self.service, self.service.changes, **kwargs)

    def queries(self, ** kwargs):
        return ServiceCaller.call(self.service, self.service.queries, **kwargs)

    def find(self, query, **kwargs):
        return ServiceCaller.call(self.service, self.service.find, data=query, **kwargs)



import sys
import os
from shutil import rmtree
from setuptools import setup, Command

NAME = 'orthanc_rest_client'

# Convert MD to RST
try:
    from pypandoc import convert_file
    read_md = lambda f: convert_file(f, 'rst')
except ModuleNotFoundError:
    print('warning: pypandoc module not found, cannot covert Markdown to RST')
    read_md = open(f, 'r').read()

# Copy from kennethreitz/setup.py
here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, NAME, '__version__.py')) as f:
    exec(f.read(), about)

class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass
        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git push --tags')

setup(name = 'orthanc_rest_client',
        version = about['__version__'],
        description = 'REST client for Orthanc DICOM servers',
        long_description = read_md(os.path.join(here, 'README.md')),
        url = 'https://github.com/teffalump/orthanc-rest-client',
        author = 'teffalump',
        author_email = 'chris@teffalump.com',
        packages = ['orthanc_rest_client'],
        install_requires = ['apiron'],
        include_package_data = True,
        zip_safe = False,
        classifiers = ['Development Status :: 2 - Pre-Alpha',
                       'Programming Language :: Python :: 3',
                       'Intended Audience :: Developers',
                       'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
                       'Operating System :: OS Independent'],
        cmdclass={
            'upload': UploadCommand,
            },
        )
